@@ -1,20 +1,20 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; html-table.scm : generate HTML programmatically, with support
-;; for simple style elements. 
+;; for simple style elements.
 ;; Copyright 2000 Bill Gribble <grib@gnumatic.com>
-;; 
+;;
 ;; * 2004.06.18: David Montenegro, added gnc:html-table-get-cell
-;; 
-;; This program is free software; you can redistribute it and/or    
-;; modify it under the terms of the GNU General Public License as   
-;; published by the Free Software Foundation; either version 2 of   
-;; the License, or (at your option) any later version.              
-;;                                                                  
-;; This program is distributed in the hope that it will be useful,  
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
-;; GNU General Public License for more details.                     
-;;                                                                  
+;;
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
 ;; You should have received a copy of the GNU General Public License
 ;; along with this program; if not, contact:
 ;;
@@ -23,17 +23,90 @@
 ;; Boston, MA  02110-1301,  USA       gnu@gnu.org
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-module (gnucash report html-table))
+
 (use-modules (srfi srfi-2))
 (use-modules (srfi srfi-9))
+(use-modules (gnucash engine))
+(use-modules (gnucash report html-style-info))
+(use-modules (gnucash report html-document))
+(use-modules (gnucash report report-utilities))
+
+(export <html-table>)
+(export gnc:html-table?)
+(export <html-table-cell>)
+(export gnc:make-html-table-cell-internal)
+(export gnc:make-html-table-cell)
+(export gnc:make-html-table-cell/size)
+(export gnc:make-html-table-cell/markup)
+(export gnc:make-html-table-cell/size/markup)
+(export gnc:make-html-table-header-cell)
+(export gnc:make-html-table-header-cell/markup)
+(export gnc:make-html-table-header-cell/size)
+(export gnc:make-html-table-cell/min-width)
+(export gnc:html-table-cell?)
+(export gnc:html-table-cell-rowspan)
+(export gnc:html-table-cell-set-rowspan!)
+(export gnc:html-table-cell-colspan)
+(export gnc:html-table-cell-set-colspan!)
+(export gnc:html-table-cell-tag)
+(export gnc:html-table-cell-set-tag!)
+(export gnc:html-table-cell-data)
+(export gnc:html-table-cell-set-data-internal!)
+(export gnc:html-table-cell-style)
+(export gnc:html-table-cell-set-style-internal!)
+(export gnc:html-table-cell-set-style!)
+(export gnc:html-table-cell-append-objects!)
+(export gnc:html-table-cell-render)
+(export gnc:make-html-table-internal)
+(export gnc:make-html-table)
+(export gnc:html-table-data)
+(export gnc:html-table-set-data!)
+(export gnc:html-table-caption)
+(export gnc:html-table-set-caption!)
+(export gnc:html-table-set-col-headers!)
+(export gnc:html-table-multirow-col-headers)
+(export gnc:html-table-set-multirow-col-headers!)
+(export gnc:html-table-style)
+(export gnc:html-table-set-style-internal!)
+(export gnc:html-table-row-styles)
+(export gnc:html-table-set-row-styles!)
+(export gnc:html-table-row-markup-table)
+(export gnc:html-table-row-markup)
+(export gnc:html-table-set-row-markup-table!)
+(export gnc:html-table-set-row-markup!)
+(export gnc:html-table-col-styles)
+(export gnc:html-table-set-col-styles!)
+(export gnc:html-table-col-headers-style)
+(export gnc:html-table-set-col-headers-style!)
+(export gnc:html-table-row-headers-style)
+(export gnc:html-table-set-row-headers-style!)
+(export gnc:html-table-set-last-row-style!)
+(export gnc:html-table-set-style!)
+(export gnc:html-table-set-col-style!)
+(export gnc:html-table-set-row-style!)
+(export gnc:html-table-row-style)
+(export gnc:html-table-col-style)
+(export gnc:html-table-num-rows)
+(export gnc:html-table-set-num-rows-internal!)
+(export gnc:html-table-num-columns)
+(export gnc:html-table-append-row/markup!)
+(export gnc:html-table-prepend-row/markup!)
+(export gnc:html-table-append-row!)
+(export gnc:html-table-prepend-row!)
+(export gnc:html-table-get-cell)
+(export gnc:html-table-set-cell!)
+(export gnc:html-table-set-cell/tag!)
+(export gnc:html-table-render)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; 
+;;
 ;; NB: In this code, "markup" and "/markup" *do not* refer to
 ;; style information.  Rather, they let you override the tag
 ;; associated with an html-table row or cell.  Style
 ;; information is stored in addition to this "markup" (in
 ;; an entirely different record field).
-;; 
+;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define-record-type <html-table>
@@ -98,7 +171,7 @@
 (define gnc:html-table-cell-set-style-internal! html-table-cell-set-style!)
 
 (define (gnc:make-html-table-cell . objects)
-  (gnc:make-html-table-cell-internal 1 1 "td" objects 
+  (gnc:make-html-table-cell-internal 1 1 "td" objects
                                      (gnc:make-html-style-table)))
 
 (define (gnc:make-html-table-cell/size rowspan colspan . objects)
@@ -106,7 +179,7 @@
                                      objects (gnc:make-html-style-table)))
 
 (define (gnc:make-html-table-cell/markup markup . objects)
-  (gnc:make-html-table-cell-internal 1 1 markup objects 
+  (gnc:make-html-table-cell-internal 1 1 markup objects
                                      (gnc:make-html-style-table)))
 
 (define (gnc:make-html-table-cell/size/markup rowspan colspan markup . objects)
@@ -120,11 +193,11 @@
     cell))
 
 (define (gnc:make-html-table-header-cell . objects)
-  (gnc:make-html-table-cell-internal 1 1 "th" objects 
+  (gnc:make-html-table-cell-internal 1 1 "th" objects
                                      (gnc:make-html-style-table)))
 
 (define (gnc:make-html-table-header-cell/markup markup . objects)
-  (gnc:make-html-table-cell-internal 1 1 markup objects 
+  (gnc:make-html-table-cell-internal 1 1 markup objects
                                      (gnc:make-html-style-table)))
 
 (define (gnc:make-html-table-header-cell/size rowspan colspan . objects)
@@ -139,7 +212,7 @@
     (gnc:html-style-table-set! styletable tag newstyle)))
 
 (define (gnc:html-table-cell-append-objects! cell . objects)
-  (gnc:html-table-cell-set-data-internal! 
+  (gnc:html-table-cell-set-data-internal!
    cell (append (gnc:html-table-cell-data cell) objects)))
 
 (define (gnc:html-table-cell-render cell doc)
@@ -180,14 +253,14 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (gnc:make-html-table)
-  (gnc:make-html-table-internal 
-   #f                    ;; col-headers 
-   #f                    ;; row-headers 
-   #f                    ;; caption 
+  (gnc:make-html-table-internal
+   #f                    ;; col-headers
+   #f                    ;; row-headers
+   #f                    ;; caption
    '()                   ;; data (stored in reverse row-major order)
    0                     ;; num-rows
    (gnc:make-html-style-table) ;; style
-   (make-hash-table 21)  ;; hash of col number to col-style 
+   (make-hash-table 21)  ;; hash of col number to col-style
    (make-hash-table 21)  ;; hash of row number to row-style
    (make-hash-table 21)  ;; hash of row number to row markup
    (gnc:make-html-style-table) ;; col-headers-style
@@ -264,11 +337,11 @@
 (define (gnc:html-table-prepend-row/markup! table markup newrow)
   (gnc:html-table-prepend-row! table newrow)
   (gnc:html-table-set-row-markup! table 0 markup))
-    
+
 
 (define (gnc:html-table-append-row! table newrow)
   (let* ((current-num-rows (gnc:html-table-num-rows table))
-	 (new-num-rows (1+ current-num-rows)))
+         (new-num-rows (1+ current-num-rows)))
     (gnc:html-table-set-num-rows-internal! table new-num-rows)
     (gnc:html-table-set-data! table
                               (cons (if (list? newrow) newrow (list newrow))
@@ -338,28 +411,6 @@
     (gnc:html-table-cell-set-tag! tc tag)
     (gnc:html-table-set-cell-datum! table row col tc)))
 
-(define (gnc:html-table-append-column! table newcol)
-  (define width (apply max (cons 0 (map length (gnc:html-table-data table)))))
-  (define (add-fn a b) (list-set-safe! b width a))
-  (let lp ((newcol newcol)
-           (olddata (reverse (gnc:html-table-data table)))
-           (res '())
-           (numrows 0))
-    (cond
-     ((null? newcol)
-      (gnc:html-table-set-num-rows-internal! table numrows)
-      (gnc:html-table-set-data! table res))
-     ((null? olddata)
-      (lp (cdr newcol)
-          '()
-          (cons (add-fn (car newcol) '()) res)
-          (1+ numrows)))
-     (else
-      (lp (cdr newcol)
-          (cdr olddata)
-          (cons (add-fn (car newcol) (car olddata)) res)
-          (1+ numrows))))))
-
 (define (gnc:html-table-render table doc)
   (let* ((retval '())
          (push (lambda (l) (set! retval (cons l retval)))))
