@@ -1,19 +1,19 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; account-piecharts.scm: shows piechart of accounts
-;;  
-;; By Robert Merkel (rgmerk@mira.net) 
+;;
+;; By Robert Merkel (rgmerk@mira.net)
 ;; and Christian Stimming <stimming@tu-harburg.de>
 ;;
-;; This program is free software; you can redistribute it and/or    
-;; modify it under the terms of the GNU General Public License as   
-;; published by the Free Software Foundation; either version 2 of   
-;; the License, or (at your option) any later version.              
-;;                                                                  
-;; This program is distributed in the hope that it will be useful,  
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
-;; GNU General Public License for more details.                     
-;;                                                                  
+;; This program is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU General Public License as
+;; published by the Free Software Foundation; either version 2 of
+;; the License, or (at your option) any later version.
+;;
+;; This program is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+;;
 ;; You should have received a copy of the GNU General Public License
 ;; along with this program; if not, contact:
 ;;
@@ -43,13 +43,13 @@
 ;; The menu statusbar tips.
 (define menutip-income
   (N_ "Shows a piechart with the Income per given time interval"))
-(define menutip-expense 
+(define menutip-expense
   (N_ "Shows a piechart with the Expenses per given time interval"))
-(define menutip-assets 
+(define menutip-assets
   (N_ "Shows a piechart with the Assets balance at a given time"))
 (define menutip-securities
   (N_ "Shows a piechart with distribution of assets over securities"))
-(define menutip-liabilities 
+(define menutip-liabilities
   (N_ "Shows a piechart with the Liabilities \
 balance at a given time"))
 
@@ -85,10 +85,7 @@ balance at a given time"))
 ;; is the list of account types that the account selection option
 ;; accepts.
 (define (options-generator account-types do-intervals? depth-based?)
-  (let* ((options (gnc:new-options)) 
-         (add-option 
-          (lambda (new-option)
-            (gnc:register-option options new-option))))
+  (let* ((options (gnc-new-optiondb)))
 
     (if do-intervals?
         (gnc:options-add-date-interval!
@@ -98,39 +95,31 @@ balance at a given time"))
          options gnc:pagename-general
          optname-to-date "a"))
 
-    (gnc:options-add-currency! 
+    (gnc:options-add-currency!
      options gnc:pagename-general optname-report-currency "b")
-    
-    (gnc:options-add-price-source! 
+
+    (gnc:options-add-price-source!
      options gnc:pagename-general
      optname-price-source "c" 'pricedb-nearest)
 
     (if do-intervals?
-        (add-option
-         (gnc:make-multichoice-option
+        (gnc-register-multichoice-option options
           gnc:pagename-general optname-averaging
           "f" opthelp-averaging
-          'None
+          "None"
           (list (vector 'None (N_ "No Averaging"))
                 (vector 'YearDelta (N_ "Yearly"))
                 (vector 'MonthDelta (N_ "Monthly"))
-                (vector 'WeekDelta (N_ "Weekly"))))))
+                (vector 'WeekDelta (N_ "Weekly")))))
 
-    (add-option
-     (gnc:make-account-list-option
+    (gnc-register-account-list-limited-option options
       gnc:pagename-accounts optname-accounts
       "a"
       (N_ "Report on these accounts, if chosen account level allows.")
-      (lambda ()
-        (gnc:filter-accountlist-type 
-         account-types
-         (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
-      (lambda (accounts)
-        (list #t
-              (gnc:filter-accountlist-type
-               account-types
-               accounts)))
-      #t))
+      (gnc:filter-accountlist-type
+       account-types
+         (gnc-account-get-descendants-sorted (gnc-get-current-root-account)))
+      account-types)
 
     (if depth-based?
       (gnc:options-add-account-levels!
@@ -138,38 +127,34 @@ balance at a given time"))
        (N_ "Maximum number of levels in the account tree displayed.")
        2))
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-fullname
       "a"
       (if depth-based?
         (N_ "Show the full account name in legend?")
         (N_ "Show the full security name in the legend?"))
-      #f))
+      #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-total
-      "b" (N_ "Show the total balance in legend?") #t))
+      "b" (N_ "Show the total balance in legend?") #t)
 
 
-     (add-option
-      (gnc:make-simple-boolean-option
+     (gnc-register-simple-boolean-option options
        gnc:pagename-display optname-show-percent
-       "b" (N_ "Show the percentage in legend?") #t))
+       "b" (N_ "Show the percentage in legend?") #t)
 
 
-    (add-option
-     (gnc:make-number-range-option
+    (gnc-register-number-range-option options
       gnc:pagename-display optname-slices
       "c" (N_ "Maximum number of slices in pie.") 7
-      2 24 0 1))
+      2 255 1)
 
-    (gnc:options-add-plot-size! 
-     options gnc:pagename-display 
+    (gnc:options-add-plot-size!
+     options gnc:pagename-display
      optname-plot-width optname-plot-height "d" (cons 'percent 100.0) (cons 'percent 100.0))
 
-    (gnc:options-add-sort-method! 
+    (gnc:options-add-sort-method!
      options gnc:pagename-display
      optname-sort-method "e" 'amount)
 
@@ -307,21 +292,20 @@ balance at a given time"))
 
   ;; This is a helper function for looking up option values.
   (define (get-option section name)
-    (gnc:option-value 
-     (gnc:lookup-option 
-      (gnc:report-options report-obj) section name)))
-  
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) section name))
+
   (gnc:report-starting reportname)
 
   ;; Get all options
-  (let ((to-date (gnc:time64-end-day-time 
+  (let ((to-date (gnc:time64-end-day-time
                      (gnc:date-option-absolute-time
                       (get-option gnc:pagename-general optname-to-date))))
         (from-date (if do-intervals?
-                          (gnc:time64-start-day-time 
-                           (gnc:date-option-absolute-time 
-                            (get-option gnc:pagename-general 
-					optname-from-date)))
+                          (gnc:time64-start-day-time
+                           (gnc:date-option-absolute-time
+                            (get-option gnc:pagename-general
+                                        optname-from-date)))
                           '()))
         (accounts (get-option gnc:pagename-accounts optname-accounts))
         (account-levels
@@ -329,11 +313,11 @@ balance at a given time"))
             (get-option gnc:pagename-accounts optname-levels)
             'all))
         (report-currency (get-option gnc:pagename-general
-				     optname-report-currency))
+                                     optname-report-currency))
         (price-source (get-option gnc:pagename-general
                                   optname-price-source))
-        (report-title (get-option gnc:pagename-general 
-				  gnc:optname-reportname))
+        (report-title (get-option gnc:pagename-general
+                                  gnc:optname-reportname))
         (averaging-selection (if do-intervals?
                                  (get-option gnc:pagename-general
                                              optname-averaging)
@@ -343,14 +327,14 @@ balance at a given time"))
         (show-total? (get-option gnc:pagename-display optname-show-total))
         (show-percent? (get-option gnc:pagename-display optname-show-percent))
         (max-slices (inexact->exact
-		     (get-option gnc:pagename-display optname-slices)))
+                     (get-option gnc:pagename-display optname-slices)))
         (height (get-option gnc:pagename-display optname-plot-height))
         (width (get-option gnc:pagename-display optname-plot-width))
-	(sort-method (get-option gnc:pagename-display optname-sort-method))
+        (sort-method (get-option gnc:pagename-display optname-sort-method))
 
         (document (gnc:make-html-document))
         (chart (gnc:make-html-chart))
-        (topl-accounts (gnc:filter-accountlist-type 
+        (topl-accounts (gnc:filter-accountlist-type
                         account-types
                         (gnc-account-get-children-sorted
                          (gnc-get-current-root-account)))))
@@ -359,7 +343,7 @@ balance at a given time"))
     ;; selection option.
     (define (show-acct? a)
       (member a accounts))
-    
+
     ;; Calculates the net balance (profit or loss) of an account
     ;; over the selected reporting period. If subaccts? == #t, all
     ;; subaccount's balances are included as well. Returns a
@@ -372,7 +356,7 @@ balance at a given time"))
            account to-date subaccts?)))
 
     ;; Define more helper variables.
-    (let* ((exchange-fn (gnc:case-exchange-fn 
+    (let* ((exchange-fn (gnc:case-exchange-fn
                          price-source report-currency to-date))
            (tree-depth (if (equal? account-levels 'all)
                            (gnc:get-current-account-tree-depth)
@@ -426,7 +410,7 @@ balance at a given time"))
         (collector->amount (profit-fn a subaccts?)))
 
       (define (count-accounts current-depth accts)
-	(if (< current-depth tree-depth)
+        (if (< current-depth tree-depth)
             (let iter ((sum 0)
                        (remaining accts))
               (if (null? remaining)
@@ -436,11 +420,11 @@ balance at a given time"))
                          (subaccts (count-accounts (1+ current-depth)
                                                    (gnc-account-get-children cur))))
                     (iter (+ sum (1+ subaccts)) tail))))
-	    (length (filter show-acct? accts))))
+            (length (filter show-acct? accts))))
 
       ;; Get base data to be plotted.
       (define work-to-do (lambda () (count-accounts 1 topl-accounts)))
- 
+
       (define base-data (lambda ()
         (get-data account-balance show-acct? work-to-do tree-depth
                   0 1 topl-accounts)))
@@ -450,15 +434,15 @@ balance at a given time"))
                (if reverse-balance?
                    (cons (- (car pair)) (cdr pair))
                    pair))
-	     combined))
+             combined))
 
       ;; Now do the work here.
 
       (if (not (null? accounts))
           (begin
             (set! combined
-		  (sort (filter (lambda (pair) (not (>= 0.0 (car pair))))
-				(fix-signs (cdr (base-data))))
+                  (sort (filter (lambda (pair) (not (>= 0.0 (car pair))))
+                                (fix-signs (cdr (base-data))))
                         (sort-comparator sort-method show-fullname?)))
 
             ;; if too many slices, condense them to an 'other' slice
@@ -478,15 +462,14 @@ balance at a given time"))
                       (gnc:options-copy-values (gnc:report-options report-obj)
                                                options)
                       ;; and set the destination accounts
-                      (gnc:option-set-value
-                       (gnc:lookup-option options gnc:pagename-accounts
-                                          optname-accounts)
+                      (gnc-set-option
+                       options gnc:pagename-accounts optname-accounts
                        (map cadr finish))
                       (set! id (gnc:make-report report-guid options))
                       ;; set the URL.
                       (set! other-anchor (gnc:report-anchor-text id))))))
-            
-            (if 
+
+            (if
              (not (null? combined))
              (let ((urls (and depth-based?
                               (map
@@ -504,7 +487,7 @@ balance at a given time"))
                                              (list gnc:pagename-accounts optname-accounts
                                                    (cons acct subaccts))
                                              (list gnc:pagename-accounts optname-levels
-                                                   (+ 1 tree-depth))
+                                                   (min 6 (+ 1 tree-depth)))
                                              (list gnc:pagename-general
                                                    gnc:optname-reportname
                                                    ((if show-fullname?
@@ -579,13 +562,13 @@ balance at a given time"))
 
              (gnc:html-document-add-object!
               document
-	      (gnc:html-make-empty-data-warning
-	       report-title (gnc:report-id report-obj)))))
+              (gnc:html-make-empty-data-warning
+               report-title (gnc:report-id report-obj)))))
 
           (gnc:html-document-add-object!
            document
-	   (gnc:html-make-no-account-warning 
-	    report-title (gnc:report-id report-obj))))
+           (gnc:html-make-no-account-warning
+            report-title (gnc:report-id report-obj))))
 
       (gnc:report-finished)
       document)))
