@@ -47,13 +47,6 @@
   (split-balance-with-closing col-datum-get-split-balance-with-closing)
   (split-value-balance col-datum-get-split-value-balance))
 
-(define FOOTER-TEXT
-  (gnc:make-html-text
-   (G_ "WARNING: Foreign currency conversions, and unrealized gains
-calculations are not confirmed correct. This report may be modified
-without notice. Bug reports are very welcome at
-https://bugs.gnucash.org/")))
-
 ;; define all option's names and help text so that they are properly
 
 (define optname-startdate (N_ "Start Date"))
@@ -136,20 +129,16 @@ also show overall period profit & loss."))
 
 ;; options generator
 (define (multicol-report-options-generator report-type)
-  (let* ((options (gnc:new-options))
-         (add-option
-          (lambda (new-option)
-            (gnc:register-option options new-option))))
+  (let* ((options (gnc-new-optiondb)))
 
     ;; date at which to report balance
     (gnc:options-add-date-interval!
      options gnc:pagename-general optname-startdate optname-enddate "c")
 
-    (add-option
-     (gnc:make-multichoice-callback-option
+    (gnc-register-multichoice-callback-option options
       gnc:pagename-general optname-period
       "c2" opthelp-period
-      'disabled
+      "disabled"
       (list
        (vector 'disabled (G_ "Disabled"))
        (vector 'YearDelta (G_ "One Year"))
@@ -158,67 +147,59 @@ also show overall period profit & loss."))
        (vector 'MonthDelta (G_ "One Month"))
        (vector 'TwoWeekDelta (G_ "Two Weeks"))
        (vector 'WeekDelta (G_ "One Week")))
-      #f
       (lambda (x)
         (let ((x (not (eq? x 'disabled))))
-          (gnc-option-db-set-option-selectable-by-name
+          (gnc-optiondb-set-option-selectable-by-name
            options
            gnc:pagename-general optname-disable-amount-indent
            (not x))
-          (gnc-option-db-set-option-selectable-by-name
+          (gnc-optiondb-set-option-selectable-by-name
            options
            gnc:pagename-general optname-dual-columns
            (not x))
-          (gnc-option-db-set-option-selectable-by-name
+          (gnc-optiondb-set-option-selectable-by-name
            options gnc:pagename-general optname-reverse-chrono x)
           (case report-type
             ((balsheet)
-             (gnc-option-db-set-option-selectable-by-name
+             (gnc-optiondb-set-option-selectable-by-name
               options gnc:pagename-general optname-include-chart x)
 
-             (gnc-option-db-set-option-selectable-by-name
+             (gnc-optiondb-set-option-selectable-by-name
               options gnc:pagename-general optname-startdate x))
 
             ((pnl)
-             (gnc-option-db-set-option-selectable-by-name
-              options gnc:pagename-general optname-include-overall-period x)))))))
+             (gnc-optiondb-set-option-selectable-by-name
+              options gnc:pagename-general optname-include-overall-period x))))))
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-disable-amount-indent
-      "c3" opthelp-disable-amount-indent #f))
+      "c3" opthelp-disable-amount-indent #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-include-chart
-      "c5" opthelp-include-chart #f))
+      "c5" opthelp-include-chart #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-dual-columns
-      "c4" opthelp-dual-columns #t))
+      "c4" opthelp-dual-columns #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-reverse-chrono
-      "c5" opthelp-reverse-chrono #t))
+      "c5" opthelp-reverse-chrono #t)
 
-    (add-option
-     (gnc:make-multichoice-option
+    (gnc-register-multichoice-option options
       gnc:pagename-general optname-options-summary
       "d" opthelp-options-summary
-      'never
+      "never"
       (list (vector 'always (G_ "Always"))
-            (vector 'never (G_ "Never")))))
+            (vector 'never (G_ "Never"))))
 
     ;; accounts to work on
-    (add-option
-     (gnc:make-account-list-option
+    (gnc-register-account-list-option options
       gnc:pagename-accounts optname-accounts
       "a"
       opthelp-accounts
-      (lambda ()
-        (gnc:filter-accountlist-type
+      (gnc:filter-accountlist-type
          (list ACCT-TYPE-BANK ACCT-TYPE-CASH ACCT-TYPE-CREDIT
                ACCT-TYPE-ASSET ACCT-TYPE-LIABILITY
                ACCT-TYPE-STOCK ACCT-TYPE-MUTUAL ACCT-TYPE-CURRENCY
@@ -226,7 +207,6 @@ also show overall period profit & loss."))
                ACCT-TYPE-EQUITY ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE
                ACCT-TYPE-TRADING)
          (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
-      #f #t))
 
     ;; the depth-limit option is not well debugged; it may be better
     ;; to disable it altogether
@@ -235,19 +215,18 @@ also show overall period profit & loss."))
      "b" opthelp-depth-limit 'all)
 
     ;; all about currencies
-    (add-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       pagename-commodities optname-common-currency
-      "b" opthelp-common-currency #f #f
+      "b" opthelp-common-currency #f
       (lambda (x)
         (for-each
          (lambda (optname)
-           (gnc-option-db-set-option-selectable-by-name
+           (gnc-optiondb-set-option-selectable-by-name
             options pagename-commodities optname x))
          (list optname-report-commodity
                optname-show-rates
                optname-show-foreign
-               optname-price-source)))))
+               optname-price-source))))
 
     (gnc:options-add-currency!
      options pagename-commodities
@@ -257,62 +236,51 @@ also show overall period profit & loss."))
      options pagename-commodities
      optname-price-source "d" 'pricedb-nearest)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-foreign
-      "e" opthelp-show-foreign #t))
+      "e" opthelp-show-foreign #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-rates
-      "f" opthelp-show-rates #t))
+      "f" opthelp-show-rates #t)
 
     ;; what to show for zero-balance accounts
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-zb-accts
-      "a" opthelp-show-zb-accts #t))
+      "a" opthelp-show-zb-accts #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-omit-zb-bals
-      "b" opthelp-omit-zb-bals #f))
+      "b" opthelp-omit-zb-bals #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-parent-balance-mode
-      "c" opthelp-parent-balance-mode #t))
+      "c" opthelp-parent-balance-mode #t)
 
     ;; some detailed formatting options
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-account-links
-      "e" opthelp-account-links #t))
+      "e" opthelp-account-links #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-amount-links
-      "e5" opthelp-amount-links #t))
+      "e5" opthelp-amount-links #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-account-full-name
-      "f" opthelp-account-full-name #f))
+      "f" opthelp-account-full-name #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
-      gnc:pagename-display optname-label-sections "g" opthelp-label-sections #t))
+    (gnc-register-simple-boolean-option options
+      gnc:pagename-display optname-label-sections "g" opthelp-label-sections #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
-      gnc:pagename-display optname-total-sections "h" opthelp-total-sections #t))
+    (gnc-register-simple-boolean-option options
+      gnc:pagename-display optname-total-sections "h" opthelp-total-sections #t)
 
     (when (eq? report-type 'pnl)
       ;; include overall period column?
-      (add-option
-       (gnc:make-simple-boolean-option
+      (gnc-register-simple-boolean-option options
         gnc:pagename-general optname-include-overall-period
-        "c6" opthelp-include-overall-period #f)))
+        "c6" opthelp-include-overall-period #f))
 
     (gnc:options-set-default-section options gnc:pagename-general)
 
@@ -393,6 +361,9 @@ also show overall period profit & loss."))
                              (gnc:make-html-table-cell/size
                               1 (if disable-account-indent? 1 (- maxindent indent))
                               label)))
+           ;; ensure the account name cell doesn't wrap
+           (_ (gnc:html-table-cell-set-style! account-cell "text-cell"
+                                              'attribute '("style" "white-space:nowrap;")))
            (row (append
                  (if disable-account-indent?
                      '()
@@ -440,13 +411,13 @@ also show overall period profit & loss."))
            (lp rest
                (let ((converted (and show-orig-cur? convert-curr-fn
                                      (convert-curr-fn monetary col-datum))))
-                 (cons* (gnc:html-markup-br)
-                        (if anchor
+                 (cons* (if anchor
                             (gnc:html-markup-anchor anchor (or converted monetary))
                             (or converted monetary))
                         (if converted
                             (gnc:html-markup-i (gnc:html-markup "small" monetary " "))
                             "")
+                        (if (null? accum) "" (gnc:html-markup-br))
                         accum)))))))
 
   (define (account->depth acc)
@@ -651,9 +622,8 @@ also show overall period profit & loss."))
 
 (define (multicol-report-renderer report-obj report-type)
   (define (get-option pagename optname)
-    (gnc:option-value
-     (gnc:lookup-option
-      (gnc:report-options report-obj) pagename optname)))
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) pagename optname))
 
   (gnc:report-starting (get-option gnc:pagename-general gnc:optname-reportname))
 
@@ -769,11 +739,11 @@ also show overall period profit & loss."))
               (cons acc (map col-datum-get-split-balance-with-closing cols-data))))
            accounts-cols-data))
 
+         ;; generate an exchange-fn
          (exchange-fn (and common-currency
-                           (gnc:case-exchange-time-fn
-                            price-source common-currency
-                            (map xaccAccountGetCommodity accounts) enddate
-                            #f #f)))
+                           (gnc:case-exchange-time-fn price-source common-currency
+                                                      (gnc:accounts-get-commodities accounts #f)
+                                                      #f #f #f)))
 
          ;; from col-idx, find effective date to retrieve pricedb
          ;; entry or to limit transactions to calculate average-cost
@@ -799,9 +769,8 @@ also show overall period profit & loss."))
                        (gnc:gnc-monetary-commodity monetary)
                        common-currency))
                  (has-price? (gnc:gnc-monetary-commodity monetary))
-                 (exchange-fn
-                  monetary common-currency
-                  (col-idx->price-date col-idx)))))
+                 (let ((col-date (col-idx->price-date col-idx)))
+                   (exchange-fn monetary common-currency col-date)))))
 
          ;; the following function generates an gnc:html-text object
          ;; to dump exchange rate for a particular column. From the
@@ -810,7 +779,8 @@ also show overall period profit & loss."))
          ;; missing price, say so.
          (get-exchange-rates-fn
           (lambda (accounts col-idx)
-            (let ((commodities (gnc:accounts-get-commodities accounts common-currency))
+            (let ((commodities (gnc:accounts-get-commodities-sorted
+                                accounts common-currency))
                   (cell (gnc:make-html-text)))
               (for-each
                (lambda (commodity)
@@ -1116,7 +1086,7 @@ also show overall period profit & loss."))
 
         (if (and common-currency show-rates?)
             (add-to-table multicol-table-right (G_ "Exchange Rates")
-                          asset-liability
+                          (append asset-liability equity-accounts)
                           #:get-col-header-fn get-exchange-rates-fn
                           #:show-accounts? #f
                           #:show-total? #f))
@@ -1279,9 +1249,6 @@ also show overall period profit & loss."))
       (gnc:html-document-add-object!
        doc multicol-table))
 
-    (gnc:html-document-add-object!
-     doc FOOTER-TEXT)
-
     (gnc:report-finished)
     ;; (gnc:html-document-set-style-text!
     ;;  doc " table, td{ border-width: 1px; border-style:solid; border-color: lightgray; border-collapse: collapse}")
@@ -1294,7 +1261,7 @@ also show overall period profit & loss."))
  'version 1
  'name balsheet-reportname
  'report-guid "065d5d5a77ba11e8b31e83ada73c5eea"
- 'menu-path (list gnc:menuname-experimental)
+ 'menu-path (list gnc:menuname-asset-liability)
  'options-generator (lambda () (multicol-report-options-generator 'balsheet))
  'renderer (lambda (rpt) (multicol-report-renderer rpt 'balsheet)))
 
@@ -1302,7 +1269,7 @@ also show overall period profit & loss."))
  'version 1
  'name pnl-reportname
  'report-guid "0e94fd0277ba11e8825d43e27232c9d4"
- 'menu-path (list gnc:menuname-experimental)
+ 'menu-path (list gnc:menuname-income-expense)
  'options-generator (lambda () (multicol-report-options-generator 'pnl))
  'renderer (lambda (rpt) (multicol-report-renderer rpt 'pnl)))
 
