@@ -40,7 +40,7 @@
 .entries-table > table { width: 100% }
 .company-table > table * { padding: 0px; }
 .client-table > table * { padding: 0px; }
-.invoice-details-table > table * { padding: 0px; }
+.invoice-details-table > table * { padding: 0px; text-indent: 0.2em; }
 @media print { .main-table > table { width: 100%; }}
 ")
 
@@ -69,8 +69,7 @@
 
 (define (build-column-used options)
   (define (opt-val section name)
-    (gnc:option-value
-     (gnc:lookup-option options section name)))
+    (gnc-optiondb-lookup-value options section name))
   (vector
    (opt-val "Display Columns" "Date")
    (opt-val "Display Columns" "Description")
@@ -108,15 +107,235 @@
       (string-append (gnc:default-html-gnc-numeric-renderer numeric #f) " " (G_ "%"))
       (gnc:make-gnc-monetary currency numeric)))
 
+
+(define (make-client-table options)
+
+  (define (get-orders invoice)
+    (fold
+     (lambda (a b)
+       (let ((order (gncEntryGetOrder a)))
+         (if (member order b) b (cons order b))))
+     '() (gncInvoiceGetEntries invoice)))
+
+  (define (opt-val section name)
+    (gnc-optiondb-lookup-value options section name))
+
+  ;; this is a single-column table.
+  (let* ((invoice (opt-val gnc:pagename-general gnc:optname-invoice-number))
+         (owner (gncInvoiceGetOwner invoice))
+         (references? (opt-val "Display" "References"))
+         (orders (if references? (get-orders invoice) '()))
+         (table (gnc:make-html-table)))
+
+    (gnc:html-table-append-row! table
+                                (list
+                                 (gnc:make-html-div/markup
+                                  "maybe-align-right client-name"
+                                  (gnc:owner-get-name-dep owner))))
+
+    (gnc:html-table-append-row! table
+                                (list
+                                 (gnc:make-html-div/markup
+                                  "maybe-align-right client-address"
+                                  (multiline-to-html-text
+                                   (gnc:owner-get-address-dep owner)))))
+
+    (if (opt-val "Display" "Invoice owner ID")
+        (gnc:html-table-append-row! table
+                                    (list
+                                     (gnc:make-html-div/markup
+                                      "maybe-align-right client-id"
+                                      (multiline-to-html-text
+                                       (gnc:owner-get-owner-id owner))))))
+
+    (for-each
+     (lambda (order)
+       (let ((reference (gncOrderGetReference order)))
+         (if (and reference (not (string-null? reference)))
+             (gnc:html-table-append-row! table
+                                         (list (string-append
+                                                (G_ "REF") " "
+                                                reference))))))
+     orders)
+
+    (gnc:make-html-div/markup "client-table" table)))
+
+(define (make-company-table options)
+
+  (define (opt-val section name)
+    (gnc-optiondb-lookup-value options section name))
+
+  ;; single-column table. my name, address, and printdate
+  (let* ((table (gnc:make-html-table))
+         (book (gncInvoiceGetBook (opt-val gnc:pagename-general gnc:optname-invoice-number)))
+         (name (gnc:company-info book gnc:*company-name*))
+         (addy (gnc:company-info book gnc:*company-addy*))
+         (phone (gnc:company-info book gnc:*company-phone*))
+         (fax (gnc:company-info book gnc:*company-fax*))
+         (email (gnc:company-info book gnc:*company-email*))
+         (url (gnc:company-info book gnc:*company-url*))
+         (taxnr (gnc:book-get-option-value book gnc:*tax-label* gnc:*tax-nr-label*))
+         (taxid (gnc:company-info book gnc:*company-id*)))
+
+    (if (and name (not (string-null? name)))
+        (gnc:html-table-append-row! table (list
+                                           (gnc:make-html-div/markup
+                                            "maybe-align-right company-name" name))))
+
+    (if (and addy (not (string-null? addy)))
+        (gnc:html-table-append-row! table (list
+                                           (gnc:make-html-div/markup
+                                            "maybe-align-right company-address" (multiline-to-html-text addy)))))
+
+    (if (and phone (not (string-null? phone)))
+        (gnc:html-table-append-row! table (list
+                                           (gnc:make-html-div/markup
+                                           "maybe-align-right company-phone" phone))))
+
+    (if (and fax (not (string-null? fax)))
+        (gnc:html-table-append-row! table (list
+                                           (gnc:make-html-div/markup
+                                            "maybe-align-right company-fax" fax))))
+
+    (if (and email (not (string-null? email)))
+        (gnc:html-table-append-row! table (list
+                                           (gnc:make-html-div/markup
+                                            "maybe-align-right company-email" email))))
+
+    (if (and url (not (string-null? url)))
+        (gnc:html-table-append-row! table (list
+                                           (gnc:make-html-div/markup
+                                            "maybe-align-right company-url" url))))
+
+    (if (and taxid (not (string-null? taxid)))
+        (gnc:html-table-append-row! table (list
+                                           (gnc:make-html-div/markup
+                                            "maybe-align-right company-tax-id" taxid))))
+
+    (if (and taxnr (not (string-null? taxnr)))
+        (gnc:html-table-append-row!
+         table (list (gnc:make-html-div/markup
+                      "maybe-align-right company-tax-nr" taxnr))))
+
+    (gnc:make-html-div/markup "company-table" table)))
+
+
+(define (make-date-row label date date-format)
+  (list
+   (string-append label ":")
+   (gnc:make-html-div/markup
+    "div-align-right"
+    (gnc-print-time64 date date-format))))
+
+(define (make-invoice-details-table options)
+  ;; dual-column. invoice date/due, billingID, terms, job name/number
+  (define (opt-val section name)
+    (gnc-optiondb-lookup-value options section name))
+  (let* ((invoice-details-table (gnc:make-html-table))
+         (invoice (opt-val gnc:pagename-general gnc:optname-invoice-number))
+         (book (gncInvoiceGetBook invoice))
+         (date-format (gnc:options-fancy-date book))
+         (jobnumber (gncJobGetID (gncOwnerGetJob (gncInvoiceGetOwner invoice))))
+         (jobname (gncJobGetName (gncOwnerGetJob (gncInvoiceGetOwner invoice)))))
+
+    (if (gncInvoiceIsPosted invoice)
+
+        (begin
+          (gnc:html-table-append-row!
+           invoice-details-table
+           (make-date-row (G_ "Date") (gncInvoiceGetDatePosted invoice) date-format))
+
+          (if (opt-val "Display" "Due Date")
+              (gnc:html-table-append-row!
+               invoice-details-table
+               (make-date-row (G_ "Due Date") (gncInvoiceGetDateDue invoice) date-format))))
+
+        (gnc:html-table-append-row! invoice-details-table
+                                    (gnc:make-html-table-cell/size
+                                     1 2 (gnc:make-html-span/markup
+                                          "invoice-in-progress"
+                                          (gnc:make-html-text
+                                           (G_ "Invoice in progress…"))))))
+
+    (if (opt-val "Display" "Billing ID")
+        (let ((billing-id (gncInvoiceGetBillingID invoice)))
+          (if (and billing-id (not (string-null? billing-id)))
+              (begin
+                (gnc:html-table-append-row! invoice-details-table
+                                            (list
+                                             (G_ "Reference:")
+                                             (gnc:make-html-div/markup
+                                              "div-align-right"
+                                              (multiline-to-html-text billing-id))))
+                (gnc:html-table-append-row! invoice-details-table '())))))
+
+    (if (opt-val "Display" "Billing Terms")
+        (let* ((term (gncInvoiceGetTerms invoice))
+               (terms (gncBillTermGetDescription term)))
+          (if (and terms (not (string-null? terms)))
+              (gnc:html-table-append-row! invoice-details-table
+                                          (list
+                                           (G_ "Terms:")
+                                           (gnc:make-html-div/markup
+                                            "div-align-right"
+                                            (multiline-to-html-text terms)))))))
+
+    ;; Add job number and name to invoice if requested and if it exists
+    (if (and (opt-val "Display" "Job Details")
+             (not (string-null? jobnumber)))
+        (begin
+          (gnc:html-table-append-row! invoice-details-table
+                                      (list (G_ "Job number:")
+                                            (gnc:make-html-div/markup
+                                             "div-align-right"
+                                             jobnumber)))
+          (gnc:html-table-append-row! invoice-details-table
+                                      (list (G_ "Job name:")
+                                            (gnc:make-html-div/markup
+                                             "div-align-right"
+                                             jobname)))))
+
+    (gnc:make-html-div/markup "invoice-details-table" invoice-details-table)))
+
+(define (make-picture options)
+  (define (opt-val section name)
+    (gnc-optiondb-lookup-value options section name))
+  (let ((img-url (opt-val "Layout" "Picture Location")))
+    (gnc:make-html-div/markup
+     "picture"
+     (gnc:make-html-text
+      (gnc:html-markup-img
+       (make-file-url img-url))))))
+
+(define (make-today options)
+  (gnc:make-html-div/markup
+   "invoice-print-date" (qof-print-date (current-time))))
+
 (define layout-key-list
-  (list (cons 'client (list (cons 'text (G_ "Client or vendor name, address and ID"))))
-        (cons 'company (list (cons 'text (G_ "Company name, address and tax-ID"))))
-        (cons 'invoice (list (cons 'text (G_ "Invoice date, due date, billing ID, terms, job details"))))
-        (cons 'today (list (cons 'text (G_ "Today's date"))))
-        (cons 'picture (list (cons 'text (G_ "Picture"))))
+  (list (list 'client
+              (cons 'renderer make-client-table)
+              (cons 'text (G_ "Client or vendor name, address and ID")))
+
+        (list 'company
+              (cons 'renderer make-company-table)
+              (cons 'text (G_ "Company name, address and tax-ID")))
+
+        (list 'invoice
+              (cons 'renderer make-invoice-details-table)
+              (cons 'text (G_ "Invoice date, due date, billing ID, terms, job details")))
+
+        (list 'today
+              (cons 'renderer make-today)
+              (cons 'text (G_ "Today's date")))
+
+        (list 'picture
+              (cons 'renderer make-picture)
+              (cons 'text (G_ "Picture")))
 
         ;; Translators: "Empty space" refers to invoice header section being left blank
-        (cons 'none (list (cons 'text (G_ "Empty space"))))))
+        (list 'none
+              (cons 'renderer (const #f))
+              (cons 'text (G_ "Empty space")))))
 
 (define variant-list
   (list
@@ -142,7 +361,7 @@
 .invoice-notes { margin-top: 20px }
 .entries-table > table { min-width: 600px }"))))
 
-   (cons 'fancy-invoice (list (cons '1a 'company)
+   (cons 'fancy-invoice (list (cons '1a 'none)
                               (cons '1b 'invoice)
                               (cons '2a 'client)
                               (cons '2b 'company)
@@ -167,218 +386,178 @@
   (gnc:multiline-to-html-text str))
 
 (define (options-generator variant)
+  (let ((options (gnc-new-optiondb)))
 
-  (define gnc:*report-options* (gnc:new-options))
+    (gnc-register-invoice-option options
+                                 gnc:pagename-general
+                                 gnc:optname-invoice-number "x" "" '())
 
-  (define (gnc:register-inv-option new-option)
-    (gnc:register-option gnc:*report-options* new-option))
-
-  (gnc:register-inv-option
-   (gnc:make-invoice-option gnc:pagename-general gnc:optname-invoice-number "x" ""
-                            (lambda () '()) #f))
-
-  (gnc:register-inv-option
-   (gnc:make-string-option
+  (gnc-register-string-option options
     gnc:pagename-general (N_ "Custom Title")
     "z" (N_ "A custom string to replace Invoice, Bill or Expense Voucher.")
-    ""))
+    "")
 
-  (gnc:register-inv-option
-   (gnc:make-text-option
+  (gnc-register-text-option options
     (N_ "Layout") (N_ "CSS") "zz" (N_ "CSS code. This field specifies the CSS code \
 for styling the invoice. Please see the exported report for the CSS class names.")
-    (keylist-get-info variant-list variant 'css)))
+    (keylist-get-info variant-list variant 'css))
 
-  (gnc:register-inv-option
-   (gnc:make-pixmap-option
+  (gnc-register-pixmap-option options
     (N_ "Layout") (N_ "Picture Location") "zy" (N_ "Location for Picture")
-    ""))
+    "")
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Date")
-    "b" (N_ "Display the date?") #t))
+    "b" (N_ "Display the date?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Description")
-    "d" (N_ "Display the description?") #t))
+    "d" (N_ "Display the description?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Action")
-    "g" (N_ "Display the action?") #t))
+    "g" (N_ "Display the action?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Quantity")
-    "ha" (N_ "Display the quantity of items?") #t))
+    "ha" (N_ "Display the quantity of items?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Price")
-    "hb" (N_ "Display the price per item?") #t))
+    "hb" (N_ "Display the price per item?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Discount")
-    "k" (N_ "Display the entry's discount?") #t))
+    "k" (N_ "Display the entry's discount?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Taxable")
-    "l" (N_ "Display the entry's taxable status?") #t))
+    "l" (N_ "Display the entry's taxable status?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Tax Amount")
-    "m" (N_ "Display each entry's total total tax?") #f))
+    "m" (N_ "Display each entry's total total tax?") #f)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display Columns") (N_ "Total")
-    "n" (N_ "Display the entry's value?") #t))
+    "n" (N_ "Display the entry's value?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Due Date")
-    "c" (N_ "Display due date?") #t))
+    "c" (N_ "Display due date?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Subtotal")
-    "d" (N_ "Display the subtotals?") #t))
+    "d" (N_ "Display the subtotals?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-complex-boolean-option
+  (gnc-register-complex-boolean-option options
     (N_ "Display") (N_ "Payable to")
-    "ua1" (N_ "Display the Payable to: information.") #f #f
+    "ua1" (N_ "Display the Payable to: information.") #f
     (lambda (x)
-      (gnc-option-db-set-option-selectable-by-name
-       gnc:*report-options* "Display" "Payable to string" x))))
+      (gnc-optiondb-set-option-selectable-by-name
+       options "Display" "Payable to string" x)))
 
-  (gnc:register-inv-option
-   (gnc:make-text-option
+  (gnc-register-text-option options
     (N_ "Display") (N_ "Payable to string")
     "ua2" (N_ "The phrase for specifying to whom payments should be made.")
-    (G_ "Please make all checks payable to")))
+    (G_ "Please make all checks payable to"))
 
-  (gnc:register-inv-option
-   (gnc:make-complex-boolean-option
+  (gnc-register-complex-boolean-option options
     (N_ "Display") (N_ "Company contact")
-    "ub1" (N_ "Display the Company contact information.") #f #f
-    (lambda (x) (gnc-option-db-set-option-selectable-by-name
-                 gnc:*report-options* "Display" "Company contact string" x))))
+    "ub1" (N_ "Display the Company contact information.") #f
+    (lambda (x) (gnc-optiondb-set-option-selectable-by-name
+                 options "Display" "Company contact string" x)))
 
-  (gnc:register-inv-option
-   (gnc:make-text-option
+  (gnc-register-text-option options
     (N_ "Display") (N_ "Company contact string")
     "ub2" (N_ "The phrase used to introduce the company contact.")
-    (G_ "Please direct all enquiries to")))
+    (G_ "Please direct all enquiries to"))
 
-  (gnc:register-inv-option
-   (gnc:make-number-range-option
+  (gnc-register-number-range-option options
     (N_ "Display") (N_ "Minimum # of entries")
     "zz" (N_ "The minimum number of invoice entries to display.") 1
-    0 23 0 1))
+    0 23 1)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Use Detailed Tax Summary")
-    "o" (N_ "Display all tax categories separately (one per line) instead of one single tax line.?") #f))
+    "o" (N_ "Display all tax categories separately (one per line) instead of one single tax line.?") #f)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "References")
-    "s" (N_ "Display the invoice references?") #t))
+    "s" (N_ "Display the invoice references?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Billing Terms")
-    "t" (N_ "Display the invoice billing terms?") #t))
+    "t" (N_ "Display the invoice billing terms?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Billing ID")
-    "ta" (N_ "Display the billing id?") #t))
+    "ta" (N_ "Display the billing id?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Invoice owner ID")
-    "tam" (N_ "Display the customer/vendor id?") #f))
+    "tam" (N_ "Display the customer/vendor id?") #f)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Invoice Notes")
-    "tb" (N_ "Display the invoice notes?") #f))
+    "tb" (N_ "Display the invoice notes?") #f)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Payments")
-    "tc" (N_ "Display the payments applied to this invoice?") #t))
+    "tc" (N_ "Display the payments applied to this invoice?") #t)
 
-  (gnc:register-inv-option
-   (gnc:make-simple-boolean-option
+  (gnc-register-simple-boolean-option options
     (N_ "Display") (N_ "Job Details")
-    "td" (N_ "Display the job name for this invoice?") #f))
+    "td" (N_ "Display the job name for this invoice?") #f)
 
-  (gnc:register-inv-option
-   (gnc:make-text-option
+  (gnc-register-text-option options
     (N_ "Display") (N_ "Extra Notes")
     "u" (N_ "Extra notes to put on the invoice.")
-    (G_ "Thank you for your patronage!")))
+    (G_ "Thank you for your patronage!"))
 
-  (gnc:register-inv-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     (N_ "Layout") (N_ "Row 1 Left")
     "1a" "1st row, left"
-    (keylist-get-info variant-list variant '1a)
-    (keylist->vectorlist layout-key-list)))
+    (symbol->string (keylist-get-info variant-list variant '1a))
+    (keylist->vectorlist layout-key-list))
 
-  (gnc:register-inv-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     (N_ "Layout") (N_ "Row 1 Right")
     "1b" "1st row, right"
-    (keylist-get-info variant-list variant '1b)
-    (keylist->vectorlist layout-key-list)))
+    (symbol->string (keylist-get-info variant-list variant '1b))
+    (keylist->vectorlist layout-key-list))
 
-  (gnc:register-inv-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     (N_ "Layout") (N_ "Row 2 Left")
     "2a" "2nd row, left"
-    (keylist-get-info variant-list variant '2a)
-    (keylist->vectorlist layout-key-list)))
+    (symbol->string (keylist-get-info variant-list variant '2a))
+    (keylist->vectorlist layout-key-list))
 
-  (gnc:register-inv-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     (N_ "Layout") (N_ "Row 2 Right")
     "2b" "2nd row, right"
-    (keylist-get-info variant-list variant '2b)
-    (keylist->vectorlist layout-key-list)))
+    (symbol->string (keylist-get-info variant-list variant '2b))
+    (keylist->vectorlist layout-key-list))
 
-  (gnc:register-inv-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     (N_ "Layout") (N_ "Row 3 Left")
     "3a" "3rd row, left"
-    (keylist-get-info variant-list variant '3a)
-    (keylist->vectorlist layout-key-list)))
+    (symbol->string (keylist-get-info variant-list variant '3a))
+    (keylist->vectorlist layout-key-list))
 
-  (gnc:register-inv-option
-   (gnc:make-multichoice-option
+  (gnc-register-multichoice-option options
     (N_ "Layout") (N_ "Row 3 Right")
     "3b" "3rd row, right"
-    (keylist-get-info variant-list variant '3b)
-    (keylist->vectorlist layout-key-list)))
+    (symbol->string (keylist-get-info variant-list variant '3b))
+    (keylist->vectorlist layout-key-list))
 
-  (gnc:options-set-default-section gnc:*report-options* "General")
-
-  gnc:*report-options*)
+  (gnc:options-set-default-section options "General")
 
+  options))
 
 (define (make-entry-table invoice options cust-doc? credit-note?)
   (define (opt-val section name)
-    (gnc:option-value
-     (gnc:lookup-option options section name)))
+    (gnc-optiondb-lookup-value options section name))
 
   (let ((show-payments (opt-val "Display" "Payments"))
         (display-all-taxes (opt-val "Display" "Use Detailed Tax Summary"))
@@ -398,14 +577,13 @@ for styling the invoice. Please see the exported report for the CSS class names.
                     monetary)
                 monetary))))
 
-    (define (add-payment-row table used-columns split total-collector reverse-payments?)
+    (define (add-payment-row table used-columns currency split total-collector reverse-payments?)
       (let* ((t (xaccSplitGetParent split))
-             (currency (xaccTransGetCurrency t))
              ;; Depending on the document type, the payments may need to be sign-reversed
              (amt (gnc:make-gnc-monetary currency
                                          (if reverse-payments?
-                                             (- (xaccSplitGetValue split))
-                                             (xaccSplitGetValue split)))))
+                                             (- (xaccSplitGetAmount split))
+                                             (xaccSplitGetAmount split)))))
 
         (total-collector 'add
                          (gnc:gnc-monetary-commodity amt)
@@ -454,7 +632,7 @@ for styling the invoice. Please see the exported report for the CSS class names.
           (addif (price-col used-columns)
                  (gnc:make-html-table-cell/markup
                   "number-cell"
-                  (gnc:make-gnc-monetary
+                  (gnc:default-price-renderer
                    currency (if cust-doc?
                                 (gncEntryGetInvPrice entry)
                                 (gncEntryGetBillPrice entry)))))
@@ -552,7 +730,7 @@ for styling the invoice. Please see the exported report for the CSS class names.
                     (for-each
                      (lambda (split)
                        (if (not (equal? (xaccSplitGetParent split) txn))
-                           (add-payment-row table used-columns
+                           (add-payment-row table used-columns currency
                                             split total-collector
                                             reverse-payments?)))
                      splits)))
@@ -571,188 +749,12 @@ for styling the invoice. Please see the exported report for the CSS class names.
 
       table)))
 
-(define (make-invoice-details-table invoice options)
-  ;; dual-column. invoice date/due, billingID, terms, job name/number
-  (define (opt-val section name)
-    (gnc:option-value
-     (gnc:lookup-option options section name)))
-  (let* ((invoice-details-table (gnc:make-html-table))
-         (book (gncInvoiceGetBook invoice))
-         (date-format (gnc:options-fancy-date book))
-         (jobnumber (gncJobGetID (gncOwnerGetJob (gncInvoiceGetOwner invoice))))
-         (jobname (gncJobGetName (gncOwnerGetJob (gncInvoiceGetOwner invoice)))))
-
-    (if (gncInvoiceIsPosted invoice)
-
-        (begin
-          (gnc:html-table-append-row!
-           invoice-details-table
-           (make-date-row (G_ "Date") (gncInvoiceGetDatePosted invoice) date-format))
-
-          (if (opt-val "Display" "Due Date")
-              (gnc:html-table-append-row!
-               invoice-details-table
-               (make-date-row (G_ "Due Date") (gncInvoiceGetDateDue invoice) date-format))))
-
-        (gnc:html-table-append-row! invoice-details-table
-                                    (gnc:make-html-table-cell/size
-                                     1 2 (gnc:make-html-span/markup
-                                          "invoice-in-progress"
-                                          (gnc:make-html-text
-                                           (G_ "Invoice in progress..."))))))
-
-    (if (opt-val "Display" "Billing ID")
-        (let ((billing-id (gncInvoiceGetBillingID invoice)))
-          (if (and billing-id (not (string-null? billing-id)))
-              (begin
-                (gnc:html-table-append-row! invoice-details-table
-                                            (list
-                                             (G_ "Reference:")
-                                             (gnc:make-html-div/markup
-                                              "div-align-right"
-                                              (multiline-to-html-text billing-id))))
-                (gnc:html-table-append-row! invoice-details-table '())))))
-
-    (if (opt-val "Display" "Billing Terms")
-        (let* ((term (gncInvoiceGetTerms invoice))
-               (terms (gncBillTermGetDescription term)))
-          (if (and terms (not (string-null? terms)))
-              (gnc:html-table-append-row! invoice-details-table
-                                          (list
-                                           (G_ "Terms:")
-                                           (gnc:make-html-div/markup
-                                            "div-align-right"
-                                            (multiline-to-html-text terms)))))))
-
-    ;; Add job number and name to invoice if requested and if it exists
-    (if (and (opt-val "Display" "Job Details")
-             (not (string-null? jobnumber)))
-        (begin
-          (gnc:html-table-append-row! invoice-details-table
-                                      (list (G_ "Job number:")
-                                            (gnc:make-html-div/markup
-                                             "div-align-right"
-                                             jobnumber)))
-          (gnc:html-table-append-row! invoice-details-table
-                                      (list (G_ "Job name:")
-                                            (gnc:make-html-div/markup
-                                             "div-align-right"
-                                             jobname)))))
-    invoice-details-table))
-
-(define (make-img img-url)
-  ;; just an image
-  (gnc:make-html-text
-   (gnc:html-markup-img
-    (make-file-url img-url))))
-
-(define (make-client-table owner orders options)
-  (define (opt-val section name)
-    (gnc:option-value
-     (gnc:lookup-option options section name)))
-  ;; this is a single-column table.
-  (let ((table (gnc:make-html-table)))
-
-    (gnc:html-table-append-row! table
-                                (list
-                                 (gnc:make-html-div/markup
-                                  "maybe-align-right client-name"
-                                  (gnc:owner-get-name-dep owner))))
-
-    (gnc:html-table-append-row! table
-                                (list
-                                 (gnc:make-html-div/markup
-                                  "maybe-align-right client-address"
-                                  (multiline-to-html-text
-                                   (gnc:owner-get-address-dep owner)))))
-
-    (if (opt-val "Display" "Invoice owner ID")
-        (gnc:html-table-append-row! table
-                                    (list
-                                     (gnc:make-html-div/markup
-                                      "maybe-align-right client-id"
-                                      (multiline-to-html-text
-                                       (gnc:owner-get-owner-id owner))))))
-
-    (for-each
-     (lambda (order)
-       (let ((reference (gncOrderGetReference order)))
-         (if (and reference (not (string-null? reference)))
-             (gnc:html-table-append-row! table
-                                         (list (string-append
-                                                (G_ "REF") " "
-                                                reference))))))
-     orders)
-
-    table))
-
-(define (make-date-row label date date-format)
-  (list
-   (string-append label ":")
-   (gnc:make-html-div/markup
-    "div-align-right"
-    (gnc-print-time64 date date-format))))
-
-(define (make-company-table book)
-  ;; single-column table. my name, address, and printdate
-  (let* ((table (gnc:make-html-table))
-         (name (gnc:company-info book gnc:*company-name*))
-         (addy (gnc:company-info book gnc:*company-addy*))
-         (phone (gnc:company-info book gnc:*company-phone*))
-         (fax (gnc:company-info book gnc:*company-fax*))
-         (email (gnc:company-info book gnc:*company-email*))
-         (url (gnc:company-info book gnc:*company-url*))
-         (taxnr (gnc:option-get-value book gnc:*tax-label* gnc:*tax-nr-label*))
-         (taxid (gnc:company-info book gnc:*company-id*)))
-
-    (if (and name (not (string-null? name)))
-        (gnc:html-table-append-row! table (list
-                                           (gnc:make-html-div/markup
-                                            "maybe-align-right company-name" name))))
-
-    (if (and addy (not (string-null? addy)))
-        (gnc:html-table-append-row! table (list
-                                           (gnc:make-html-div/markup
-                                            "maybe-align-right company-address" (multiline-to-html-text addy)))))
-
-    (if (and phone (not (string-null? phone)))
-        (gnc:html-table-append-row! table (list
-                                           (gnc:make-html-div/markup
-                                           "maybe-align-right company-phone" phone))))
-
-    (if (and fax (not (string-null? fax)))
-        (gnc:html-table-append-row! table (list
-                                           (gnc:make-html-div/markup
-                                            "maybe-align-right company-fax" fax))))
-
-    (if (and email (not (string-null? email)))
-        (gnc:html-table-append-row! table (list
-                                           (gnc:make-html-div/markup
-                                            "maybe-align-right company-email" email))))
-
-    (if (and url (not (string-null? url)))
-        (gnc:html-table-append-row! table (list
-                                           (gnc:make-html-div/markup
-                                            "maybe-align-right company-url" url))))
-
-    (if (and taxid (not (string-null? taxid)))
-        (gnc:html-table-append-row! table (list
-                                           (gnc:make-html-div/markup
-                                            "maybe-align-right company-tax-id" taxid))))
-
-    (if (and taxnr (not (string-null? taxnr)))
-        (gnc:html-table-append-row!
-         table (list (gnc:make-html-div/markup
-                      "maybe-align-right company-tax-nr" taxnr))))
-
-    table))
 
 (define (reg-renderer report-obj)
   (let* ((document (gnc:make-html-document))
          (options (gnc:report-options report-obj))
-         (opt-val (lambda (section name) (gnc:option-value (gnc:lookup-option options section name))))
+         (opt-val (lambda (section name) (gnc-optiondb-lookup-value options section name)))
          (invoice (opt-val gnc:pagename-general gnc:optname-invoice-number))
-         (references? (opt-val "Display" "References"))
          (custom-title (opt-val gnc:pagename-general "Custom Title")))
 
     (if (null? invoice)
@@ -764,9 +766,7 @@ for styling the invoice. Please see the exported report for the CSS class names.
           (G_ "No valid invoice selected. Click on the Options button and select the invoice to use.")))
 
         (let* ((book (gncInvoiceGetBook invoice))
-               (owner (gncInvoiceGetOwner invoice))
                (type (gncInvoiceGetType invoice))
-               (orders (if references? (delete-duplicates (map gncEntryGetOrder (gncInvoiceGetEntries invoice))) '()))
                (cust-doc? (memv type (list GNC-INVOICE-CUST-INVOICE GNC-INVOICE-CUST-CREDIT-NOTE)))
                (credit-note? (memv type (list GNC-INVOICE-CUST-CREDIT-NOTE GNC-INVOICE-VEND-CREDIT-NOTE GNC-INVOICE-EMPL-CREDIT-NOTE)))
                (default-title (cond
@@ -781,29 +781,16 @@ for styling the invoice. Please see the exported report for the CSS class names.
                                (else
                                 (G_ "Invoice"))))
                (title (if (string-null? custom-title) default-title custom-title))
-               ;; Translators: This is the format of the invoice title.
-               ;; The first ~a is "Invoice", "Credit Note"... and the second the number.
-               ;; Replace " #" by whatever is common as number abbreviation, i.e. "~a Nr. ~a"
+               ;; Translators: This is the format of the invoice
+               ;; title.  The first ~a is one of "Invoice", "Credit
+               ;; Note", and so on and the second the number.  Replace
+               ;; " #" by whatever is common as number abbreviation,
+               ;; i.e. "~a Nr. ~a"
                (invoice-title (format #f (G_"~a #~a") title (gncInvoiceGetID invoice)))
-               (layout-lookup-table (list (cons 'none #f)
-                                          (cons 'picture (gnc:make-html-div/markup
-                                                          "picture"
-                                                          (make-img (opt-val "Layout" "Picture Location"))))
-                                          (cons 'invoice (gnc:make-html-div/markup
-                                                          "invoice-details-table"
-                                                          (make-invoice-details-table
-                                                           invoice options)))
-                                          (cons 'client (gnc:make-html-div/markup
-                                                         "client-table"
-                                                         (make-client-table
-                                                          owner orders options)))
-                                          (cons 'company (gnc:make-html-div/markup
-                                                          "company-table"
-                                                          (make-company-table book)))
-                                          (cons 'today (gnc:make-html-div/markup
-                                                        "invoice-print-date"
-                                                        (qof-print-date (current-time))))))
-               (layout-lookup (lambda (loc) (cdr (assq (opt-val "Layout" loc) layout-lookup-table)))))
+               (layout-lookup (lambda (loc)
+                                (let* ((key (opt-val "Layout" loc))
+                                       (renderer (keylist-get-info layout-key-list key 'renderer)))
+                                  (renderer options)))))
 
           (gnc:html-document-set-style-text! document (opt-val "Layout" "CSS"))
 
@@ -892,6 +879,7 @@ for styling the invoice. Please see the exported report for the CSS class names.
  'menu-path (list gnc:menuname-business-reports)
  'options-generator (lambda () (options-generator 'invoice))
  'renderer reg-renderer
+ 'hook 'invoice
  'in-menu? #t)
 
 (gnc:define-report
@@ -901,6 +889,7 @@ for styling the invoice. Please see the exported report for the CSS class names.
  'menu-path (list gnc:menuname-business-reports)
  'options-generator (lambda () (options-generator 'easy-invoice))
  'renderer reg-renderer
+ 'hook 'invoice
  'in-menu? #t)
 
 (gnc:define-report
@@ -910,5 +899,6 @@ for styling the invoice. Please see the exported report for the CSS class names.
  'menu-path (list gnc:menuname-business-reports)
  'options-generator (lambda () (options-generator 'fancy-invoice))
  'renderer reg-renderer
+ 'hook 'invoice
  'in-menu? #t)
 
